// Copyright 2011 Thomas Kamps anubis1@linux-ecke.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include "bookmarkfolder.h"

#include <QApplication>
#include <QMenu>

BookmarkFolder::BookmarkFolder() {
	m_name = "";
	m_allAction = new QAction(0);
	m_emptyAction = 0;
	m_action = new QAction(0);
	m_gotoAction = new QAction(0);
	m_action->setMenu(new QMenu());
	m_gotoAction->setMenu(new QMenu());

	m_action->setVisible(false);
	m_gotoAction->setVisible(false);
	m_allAction->setVisible(false);
}

BookmarkFolder::~BookmarkFolder() {
	//Delete all subfolders
	while (!m_subfolders.isEmpty()) {
		BookmarkFolder* f = m_subfolders.value(0);
		m_subfolders.removeAll(f);
		delete f;
	}

	//Delete all bookmarks
	while (!m_bookmarks.isEmpty()) {
		Bookmark* b = m_bookmarks.value(0);
		m_bookmarks.removeAll(b);
		delete b;
	}

	QMenu* m = m_action->menu();
	m_action->setMenu(0);
	delete m;
	delete m_action;

	m = m_gotoAction->menu();
	m_gotoAction->setMenu(0);
	delete m;
	delete m_gotoAction;

	delete m_allAction;
	if (m_emptyAction) delete m_emptyAction;
}

BookmarkFolder* BookmarkFolder::readFolder(QObject* parent, QDomElement folder, bool isRootFolder) {
	BookmarkFolder* res = new BookmarkFolder();
	res->setName(folder.attribute("name", "Root"));

	if (isRootFolder) {
		res->addEmptyAction();
		res->gotoAction()->setText(QApplication::tr("Go to"));
	}

	res->addAllAction();

	QDomElement e = folder.firstChildElement();
	while (!e.isNull()) {
		if (e.nodeName() == "bookmarkfolder") res->addFolder(readFolder(parent, e));
		if (e.nodeName() == "bookmark") res->addBookmark(Bookmark::readBookmark(parent, e));
		e = e.nextSiblingElement();
	}

	res->action()->setParent(parent);
	res->gotoAction()->setParent(parent);
	return res;
}

QDomElement BookmarkFolder::toXml(QDomDocument* doc) {
	QDomElement folder = doc->createElement("bookmarkfolder");
	folder.setAttribute("name", m_name);

	for (int i = 0; i < m_subfolders.size(); i++) {
		folder.appendChild(m_subfolders.value(i)->toXml(doc));
	}

	for (int i = 0; i < m_bookmarks.size(); i++) {
		folder.appendChild(m_bookmarks.value(i)->toXml(doc));
	}

	return folder;
}

BookmarkFolder* BookmarkFolder::getFolder(QString name) {
	if (m_name == name) return this;

	for (int i = 0; i < m_subfolders.size(); i++) {
		BookmarkFolder* res = m_subfolders.value(i)->getFolder(name);
		if (res != 0) return res;
	}

	return 0;
}

void BookmarkFolder::setName(QString name) {
	m_name = name;
	m_action->setText(name);
	m_action->setData(QVariant(name));
	m_gotoAction->setText(name);
	m_gotoAction->setData(QVariant(name));
	m_allAction->setData(QVariant("all|"+m_name));
}

void BookmarkFolder::addFolder(BookmarkFolder *folder, bool sort) {
	m_subfolders.append(folder);
	if (sort) rearrangeEntries();
	else {
		m_action->menu()->addAction(folder->action());
		m_gotoAction->menu()->addAction(folder->gotoAction());
	}
	updateVisibility();
}

void BookmarkFolder::addBookmark(Bookmark *bookmark) {
	m_bookmarks.append(bookmark);
	m_action->menu()->addAction(bookmark->action());
	m_gotoAction->menu()->addAction(bookmark->gotoAction());
	updateVisibility();
}

void BookmarkFolder::removeBookmark(Bookmark *bookmark) {
	m_bookmarks.removeAll(bookmark);
	updateVisibility();
}

void BookmarkFolder::removeFolder(BookmarkFolder *folder) {
	m_subfolders.removeAll(folder);
	updateVisibility();
}

void BookmarkFolder::addSeparator() {
	m_action->menu()->addSeparator();
}

void BookmarkFolder::addEmptyAction() {
	m_emptyAction = m_action->menu()->addAction(QIcon(":/icons/new.png"), QApplication::tr("Empty page"));
	m_emptyAction->setData("about:blank");
}

void BookmarkFolder::addAllAction() {
	m_allAction->setText(QApplication::tr("Open all"));
	m_action->menu()->addAction(m_allAction);
	m_action->menu()->addSeparator();
	m_allAction->setData(QVariant(QString("all|")+m_name));
}

int BookmarkFolder::size() {
	int res = m_bookmarks.size();
	foreach (BookmarkFolder* f, m_subfolders) {
		res += f->size();
	}

	return res;
}

void BookmarkFolder::updateVisibility() {
	int s = size();
	m_gotoAction->setVisible(s > 0);
	m_action->setVisible(s > 0);
	m_allAction->setVisible(size() > 1);

	foreach (BookmarkFolder* f, m_subfolders) {
		f->updateVisibility();
	}
}

void BookmarkFolder::rearrangeEntries() {
	//Sorts the entries: first (if there the emptypage), second "open all" thrist: the fodlers, and at last the bookmarks
	m_action->menu()->clear();
	m_gotoAction->menu()->clear();

	if (m_emptyAction) m_action->menu()->addAction(m_emptyAction);
	m_action->menu()->addAction(m_allAction);

	m_action->menu()->addSeparator();

	for (int i = 0; i < m_subfolders.size(); i++) {
		m_action->menu()->addAction(m_subfolders.value(i)->action());
		m_gotoAction->menu()->addAction(m_subfolders.value(i)->gotoAction());
	}
	for (int i = 0; i < m_bookmarks.size(); i++) {
		m_action->menu()->addAction(m_bookmarks.value(i)->action());
		m_gotoAction->menu()->addAction(m_bookmarks.value(i)->gotoAction());
	}
}
